import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.ImageIcon;
import javax.swing.BorderFactory;
import java.util.jar.JarEntry;

/**
 * @author Sebastian Gfeller, Andreas Jaggi
 * @created Jun 4, 2004
 * @version About.java,v 1.10 2004/06/27 14:28:16 ajaggi Exp
 * @copyright Sebastian Gfeller, Andreas Jaggi 2004
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 */

public class About extends JFrame implements ActionListener {
	final static int WINDOW_APROPOS = 0;
	final static int WINDOW_AUFAIT = 1;
	
	private JTextField tfNote;
	private JLabel lMessage;
	private JButton bNote;

	private boolean encoreUnEssai;
	/**
	 * On a un seul constructeur qui reoit comme argument un int, nous definissons la message  l' interieur du constructeur
	 * ce qui me parait plus simple que donner la message comme String au constructeur
	 * @param aboutType le type du fentre, a propos ou au fait.
	 */
	
	public About(int aboutType) {
		/*
		 * Les deux fentres sont trs similaires, ils ont tous deux une message dedans.
		 * La fentre au fait possede encore un JTextField et un JButton, c'est tout
		 */
		
		super();
		
		// initialiser le conteneur, les lments  ajouter
		Container cont = this.getContentPane();
		cont.setLayout(new FlowLayout()); // trop simple pour GridBag...
		
		this.lMessage = new JLabel();

		this.encoreUnEssai = true;
		
		// distinction des cas a propos - au fait
		if(aboutType == WINDOW_APROPOS) {
			this.lMessage = new JLabel(new ImageIcon(getClass().getResource("gfx/about.png")));
			this.lMessage.setBorder(BorderFactory.createEmptyBorder());
		} else {
			this.lMessage.setText("Quelle note dsirez vous  votre projet?");
		}
		
		cont.add(lMessage);
		
		if(aboutType == WINDOW_AUFAIT) { // selectionneur des notes
			this.tfNote = new JTextField(2); // on doit seulement entrer une note, a ne doit pas tre plus grand.
			this.tfNote.addActionListener(this);
			cont.add(tfNote);
			
			this.bNote = new JButton("OK");
			this.bNote.addActionListener(this);
			cont.add(bNote);
		}
		
		this.pack(); // dterminer la dimension de la fentre automatiquement
		this.centrer(); // place la fentre au milieu de l'cran
		this.setResizable(false);
	}

	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == bNote) {
			lMessage.setText(parseNote());
			

			// on efface le contenu du JTextField (e.g. on efface le contenu fausse: non numrique, pas entre 1 et 6 etc.)
			this.tfNote.setText("");
			
			// si on a encore un essai, le bouton et le TextField restent, autrement on les enleve
			if ( ! this.encoreUnEssai ) {
				Container cont = this.getContentPane();

				cont.remove(this.tfNote);
				cont.remove(this.bNote);
			}

			this.pack();
			this.repaint();
		}
	}
	
	/**
	 * change la message dans la fentre d' aprs l'entree de tfNote
	 * @return retourne si on peut encore une fois essaier d' entrer la note.
	 */
	private String parseNote() {

		String ret = "";
		
		try {
			double note = Double.parseDouble(tfNote.getText());
			
			// on a pu trouver un nombre. Est-ce que c'est entre 1 et 6?
			// On affiche des rponses differentes aux differents dsirs
			if(note >= 1 && note < 6) {
				ret = "Note spre: "+note;
				
				// On affiche la note et c'est fini
				this.encoreUnEssai = false;
			} else if (note == 6) {
				ret = "Faut pas rver!";
				// on peut encore une fois entrer un nombre
			} else { // pas de nombre valable
				ret = "Donnez un nombre entre 1 et 6";
			}
		} catch (Exception excpt) {
			ret = "Vous devez entrer un nombre";
		}
		
		return ret;
	}
	
	/**
	 * Centre la fentre dans l'cran
	 */
	private void centrer() {

		Dimension tailleEcran = Toolkit.getDefaultToolkit().getScreenSize();

		int xPos = (tailleEcran.width - this.getSize().width) / 2;
		int yPos = (tailleEcran.height - this.getSize().height) / 2;

		setLocation(xPos, yPos);
	}
}

