
/**
 * Author:      Frdric Viela
 * Date:        2004
 *
 * AStar:	Andreas Jaggi
 */


import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.io.InputStream;
import java.io.ObjectInputStream;

/**
* Reprsente le plateau de jeu au niveau type de sols
* par une matrice de Blocks
*/

public class Niveau implements Constantes {
    // La zone de jeu o afficher les Blocks
    public static ZoneJeu zoneJeu;

    // Le tableau des noms des fichiers images
    String nomFichier[];
    // Le tabeau de Blocks
    Block carte[][];
    // Le tableau des types de Block
    int type[][];
    // Le tableau des images existantes
    Image image[];


		/* fonctions utilis pour AStar (ajaggi) */

    		/**
		 * Donne les voisins  une certaine distance d'un Block 
		 * 
		 * @param b Block
		 * @param dist distance des voisins
		 *
		 * @return Tableau avec 4 Blocks
		 */
		public Block[] getVoisins ( Block b, int dist ) {
			Block voisins[] = new Block[4];
			Point p = b.getPosition();

			/* ... A l'envers,  l'endroit
			 *  l'envers,  l'endroit ... */
			if ( p.y - dist > -1 ) {
				voisins[0] = this.carte[p.x][p.y - dist];
			}

			if ( p.x + dist < NBLIGNES ) {
				voisins[1] = this.carte[p.x + dist][p.y];
			}

			if ( p.y + dist < NBCOLONNES ) {
				voisins[2] = this.carte[p.x][p.y + dist];
			}

			if ( p.x - dist > -1 ) {
				voisins[3] = this.carte[p.x - dist][p.y];
			}

			return voisins;
		}

		/**
		 * Donne les voisins direct d'un Block
		 *
		 * @param b Block
		 * @return Tableau avec 4 Blocks
		 */
		public Block[] getVoisins ( Block b ) {
			return this.getVoisins(b, 1);
		}

		/**
		 * Donne le voisin avec la distance la plus courte du dpart
		 *
		 * @param b Block
		 * @param start Dpart
		 *
		 * @return Block voisin
		 */
		public Block getPlusPetitVoisin ( Block b, Point start ) {
			/* on charge les voisins */
			Block voisins[] = this.getVoisins(b);

			/* initialsation des variables locaux */
			Block petit = voisins[0];
			double dist = Double.MAX_VALUE;
			double distVoisin;

			/* cherche la distance la plus petit de ces 4 */
			for ( int i=0; i < 4; i++ ) {
				if ( voisins[i] != null ) {
					distVoisin = voisins[i].getDistDuStart( this.carte[start.x][start.y] );

					if ( distVoisin < dist && distVoisin >= 0 ) {
						petit = voisins[i];
						dist = distVoisin;
					}
				}
			}

			return petit;
		}

		/**
		 * Fonction appelle quand une bombe a t pose.
		 * Dit au voisins qu'il sont voisin d'une bombe
		 *
		 * @param p Coordonnes de la bombe
		 * @param puissance Puissance de la bombe
		 */
		public void setBombe ( Point p, int puissance ) {

			if ( DEBUG ) {
				System.out.println("Setting bombe ("+p.x+", "+p.y+")");
			}

			Block[] voisins;

			/* on marque la bombe */
			this.carte[p.x][p.y].setCouts(COUTS_BOMBE);
			this.carte[p.x][p.y].bombe = true;

			/* et on alerte les voisins */
			for ( int i=1; i <= puissance; i++ ) {
				voisins = this.getVoisins(this.carte[p.x][p.y], i);

				for ( int j=0; j < voisins.length; j++ ) {
					voisins[j].setCouts(COUTS_BOMBEVOISIN);
					voisins[j].bombevoisin++;
				}
			}
		}

		/**
		 * Fonction appelle quand une bombe s'est xplose. 
		 * Dit a tous le Blocks voisins qu'il ne sont plus voisin de cette bombe
		 *
		 * @param p Coordonnes de la bombe
		 * @param puissance Puissance de labombe
		 */
		public void unsetBombe ( Point p, int puissance ) {

			/* variables locaux */
			Block[] voisins;

			/* on actualise le Block de la bombe */
			this.carte[p.x][p.y].bombe = false;
			this.carte[p.x][p.y].setTypeCouts();

			/* et ses voisins */
			for ( int i=1; i <= puissance; i++ ) {
				voisins = this.getVoisins(this.carte[p.x][p.y], i);

				for ( int j=0; j < voisins.length; j++ ) {
					voisins[j].bombevoisin--;
					voisins[j].setTypeCouts();
				}
			}
		}

		/**
		 * Reset les couts et les distances de tous les Blocks
		 */
		public void resetBlocks ( ) {
			for ( int i = 0; i < NBLIGNES; i++ ) {
				for ( int j = 0; j < NBCOLONNES; j++ ) {
					this.carte[i][j].resetBlock();
					this.carte[i][j].setTypeCouts();
				}
			}
		}

    /**
    * Class Constructor
    */

    public Niveau() {
        // Cre la carte et initialise le niveau 1
        setCarte(1);
        initImagesFond();
    }

    /**
    * Initialise le niveau
    * @param numNiveau numro du niveau
    */

    public void setCarte(int numNiveau) {
        initCarte(numNiveau);
        chargerImages();
        initImagesFond();
    }

    /**
    * Initialise la zone de jeu
    * @param zj zone jeu
    */

    public static void init(ZoneJeu zj) {
        zoneJeu = zj;
    }

    /**
    * Initialise le Niveau en chargeant son fichier
    * @param numNiveau numro du niveau
    */

    public void initCarte(int numNiveau) {
        InputStream fis;
        ObjectInputStream in;
        
        // Ouvre le fichier correspondant au niveau et le charge
        try {
            fis = getClass().getResourceAsStream("level/level" + numNiveau + ".lvl");
            in = new ObjectInputStream(fis);

            nomFichier = (String[])in.readObject();
            type = (int[][])in.readObject();

            in.close();
            fis.close();
        }
        catch(Exception ex) {
            new BoiteDialogue("Pas bien : Fichier (level/level"+numNiveau+".lvl) non trouv !");
            return;
        }
    }

    /**
    * Initialise le tableau de Blocks
    */

    public void initImagesFond() {
        carte = new Block[NBLIGNES][NBCOLONNES];

        // Charge les images correspondantes  la case
        for (int i = 0; i < NBLIGNES; i++)
            for(int j = 0; j < NBCOLONNES; j++)
                if (type[i][j] > 3)
                    carte[i][j] = new Block(image[1], type[i][j], j, i);
                else
                    carte[i][j] = new Block(image[type[i][j]], type[i][j], j, i);
    }


    /**
    * Initialise le tableau d'images
    */

    public void chargerImages() {
        image = new Image[nomFichier.length];

        // Charge les images correspondant au niveau
        for(int i = 0; i < nomFichier.length; i++) {
            image[i] = zoneJeu.load(nomFichier[i]);
        }
    }

    /**
    * Dtruit le Block en (i, j)
    * return false si le Block n'est pas dtruit, true sinon
    */

    public boolean detruire(int i, int j) {
        // Empeche la destruction du tour
        if ((i <= 0) || (i >= NBLIGNES) || (j <= 0) || (j > NBCOLONNES))
            return false;
        // Dtruit la case (i, j)
        else
            return carte[i][j].explose(image);
    }


    /**
    * Dit si le Block en (i, j) est destructible
    * return true si le Block est destructible, false sinon
    */

    boolean isDestructible(int j, int i) {
        // Empeche la destruction du contour
        if ((i <= 0) || (i >= NBLIGNES) || (j <= 0) || (j > NBCOLONNES))
            return false;
        // Dit si la case (i, j) est destructible
        return ((carte[i][j].type != 0) && (carte[i][j].type != 3));
    }

    /**
    * Repaint le Niveau dans le fond de la zone de jeu
    * @param g Contexte graphique du fond de la zone de jeu
    */

    public void paint(Graphics g) {
        // Repaint le fond case par case
        for(int i = 0; i < NBLIGNES; i++)
            for(int j = 0; j < NBCOLONNES; j++)
                carte[i][j].paint(g);
    }
}
