import java.awt.Point;
import java.util.Vector;


/**
 * Gre le comportement du bomberman ordinateur
 *
 * @author      Andras Jaggi, Sebastian Gfeller
 * @created     May 26, 2004
 * @version     Strategie.java,v 1.14 2004/06/26 08:52:25 ajaggi Exp
 * @copyright   Andreas Jaggi, Sebastian Gfeller 2004
 * @license     http://opensource.org/licenses/gpl-license.php GNU Public License
 */
 
public class Strategie implements Constantes {

	int modeavant;

	/**
	 * Plateau du jeu
	 */
	Plateau plateau;

	/**
	 * Le personnage ennemi
	 */
	Perso ennemi;

	/**
	 * Le personnage gr.
	 * ... J'ai les hommes  mes pieds
	 * Huit milliards potentiels ... [Noir Dsir - Homme press]
	 */
	Perso gentil;

	/**
	 * Niveau du jeu
	 */
	Niveau niveau;

	/**
	 * Instance de AStar qui gre le pathfinding
	 */
	AStar etoile;

	/**
	 * Le chemin  marcher
	 */
	Vector chemin;

	/**
	 * La position du chemin o on se trouve
	 */
	int cheminPos;

	/**
	 * La o on veut aller
	 */
	Point target;

	/**
	 * Le prochain Block du chemin au target
	 */
	Point prochainPoint;

	/**
	 * Nombre des bombes sur la carte
	 */
	int bombes;

	/**
	 * Rfrence sur la classe Intelligence qui a cre une instance de Strategie ( on en a besoin pour bouger la personnage )
	 */
	Intelligence intelligence;

	/**
	 * Constante qui indique le mode chercher l'ennemi
	 */
	final int MODE_CHERCHER = 0;

	/**
	 * Constante qui indique le mode chercher un bonus
	 */
	final int MODE_BONUS = 1;

	/**
	 * Constante qui indique le mode se cacher des bombes
	 */
	final int MODE_CACHER = 2;

	/**
	 * Indique le mode actuellement utilise
	 */
	int mode;

	/**
	 * Indique si on a besoin de chercher un Chemin
	 */
	boolean chercheChemin;

	/**
	 * Les coordonnes des bonus
	 */
	Vector bonusPoints;

	/**
	 * Variable compteur
	 */
	int bombwait;

	/**
	 * Constructeur de Strategie
	 *
	 * @param plateau       Plateau du jeu
	 * @param gentil        Personnage  grer
	 * @param ennemi        Personnage ennemi
	 * @param niveau        Niveau du jeu
	 * @param intelligence  Intelligence qui a instance cette Strategie
	 */
	public Strategie( Plateau plateau, Perso gentil, Perso ennemi, Niveau niveau, Intelligence intelligence ) {

		/* charge les variables */
		this.plateau = plateau; this.gentil = gentil;
		this.ennemi = ennemi;
		this.niveau = niveau;
		this.intelligence = intelligence;

		/* on commence par chercher l'ennemi */
		this.mode = MODE_CHERCHER;
		this.chercheChemin = true;

		/* cre une nouveau instance de AStar */
		this.etoile = new AStar(this.plateau, this.niveau);

		/* calcule un premier chemin */
		this.chemin = new Vector();

		this.etoile.calculeChemin( this.gentil.coord, this.ennemi.coord );
		this.chemin = this.etoile.getChemin();

		this.cheminPos = 0;

		/* d'abord il n'y a pas encore des bombes */
		this.bombes = 0;
		this.bonusPoints = new Vector();

		this.target = new Point(this.ennemi.coord.x, this.ennemi.coord.y);
		this.prochainPoint = this.gentil.coord;

		this.bombwait = 0;

	}


	/**
	 * Mthode centrale qui est appel tous les 80 ms par Intelligence, 
	 * gre le comportement de la personnage
	 */
	public void agir() {

		this.intelligence.stop();

		/* si on est "sur" l'ennemi, on pose une bombe */
		if ( this.compPoints(this.gentil.coord, this.ennemi.coord) ) {
			this.intelligence.poseBombe();

			/* et ensuite on va se cacher des bombes */
			/* ... au bout de la course
			 * remonte jusqu' la source ... [Noir Dsir - One Trip, One Noise] */
			this.intelligence.stop();
			this.mode = MODE_CACHER;
			this.target = this.chercheSecurite(this.gentil.coord);
			this.chercheChemin = true;
		}

		/* quelqu'un a pose une bombe ou une bombe s'est explose,
		 * (e.g. la nombre des bombes sur la carte a change)
		 * alors on calcule un nouveau chemin */
		if ( this.bombes != this.plateau.perso.bombePosees.size() ) {
			this.bombes = this.plateau.perso.bombePosees.size();
			/* on actualise les coordonnes des bonus */
			this.chercheBonus();

			/* si on et en mode se cacher des bombes, on cherche une nouvelle postition sans bombes */
			/* ... longue vie sans jamais s'craser ... [N.D] */
			if ( this.mode == MODE_CACHER ) {
				this.intelligence.stop();
				this.target = this.chercheSecurite(this.gentil.coord);
			}

			this.chercheChemin = true;
		}

		/* si on n'est pas en train de se cacher et il y a des bonus,
		 * alors on va collectionner les bonus */
		/* ... specialiste de l'enjeu ... [N.D] */
		if ( this.mode != MODE_CACHER && this.bonusPoints.size() > 0 ) {
			this.intelligence.stop();
			this.mode = MODE_BONUS;
			this.target = this.getPlusProcheBonus();
			this.chercheChemin = true;
		}

		/* Si l'ennemi a bouge, on cherche un nouveau chemin */
		if ( ( this.mode == MODE_CHERCHER ) && ( this.ennemi.coord.x != this.target.x || this.ennemi.coord.y != this.target.y ) ) {
			this.target = new Point(this.ennemi.coord.x, this.ennemi.coord.y);
			this.chercheChemin = true;
		}

		/* si on et en train de se cacher des bombes, mais il n'y plus de bombes, on change en mode chercher l'ennemi */
		//if ( this.mode == MODE_CACHER && this.gentil.bombePosees.size() == 0 ) {
		if ( this.mode == MODE_CACHER && this.plateau.perso.bombePosees.size() == 0 ) {
			this.mode = MODE_CHERCHER;
			this.target = new Point(this.ennemi.coord.x, this.ennemi.coord.y);
			this.chercheChemin = true;

			/* il faut attendre 3 tours au fin de l'explosion de la bombe, sinon on va s'craser! */
			/* ... longue attente avant de s'elancer ... [N.D] */
			this.bombwait = 3;
		}


		/* on doit chercher un nouveau chemin (seulement si on n'est pas dj sur l'arrive) */
		if ( this.chercheChemin && !this.compPoints(this.target, this.gentil.coord) ) {
			this.intelligence.stop();

			this.chercheNouveauChemin();
			this.chercheChemin = false;
		}

		/* si on ne doit pas attendre le fin d'une explosion d'une bombe,
		 * on va chez le prochain point du chemin */
		if ( this.bombwait == 0 ) {
			this.deplaceAnext();
		} else {
			this.bombwait--;
		}

		if ( DEBUG && this.mode != this.modeavant ) {
			this.modeavant = this.mode;

			switch ( this.mode ) {
				case MODE_CHERCHER:
					System.out.println("MODE_CHERCHER");
					break;
				case MODE_CACHER:
					System.out.println("MODE_CACHER");
					break;
				case MODE_BONUS:
					System.out.println("MODE_BONUS");
					break;
			}
		}
	}

	/**
	 * Compare deux java.awt.Point par leurs coordonnes
	 *
	 * @param a Point a
	 * @param b Point b
	 *
	 * @return Mmes coordonnes
	 */
	private boolean compPoints( Point a, Point b ) {
		if ( a.x == b.x && a.y == b.y ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Cherche le bonus le plus proche. 
	 * S'il n'y a pas de bonus, on tombe en mode chercher l'ennemi
	 * @return Le Bonus le plus proche
	 */
	private Point getPlusProcheBonus ( ) {
		/* cherche tous les bonus */
		this.chercheBonus();

		Point best = new Point(this.ennemi.coord.x, this.ennemi.coord.y);

		/* s'il n'y a pas de bonus, on va chercher l'ennemi */
		if ( this.bonusPoints.size() > 0 ) {
			Point actuel;
			int min = Integer.MAX_VALUE;
			int score;

			/* parcourt tous les bonus */
			for ( int i=0; i < this.bonusPoints.size(); i++ ) {
				actuel  = (Point)this.bonusPoints.elementAt(i);
				/* calcule la distance estime */
				score = Math.abs(actuel.x - this.gentil.coord.x + actuel.y - this.gentil.coord.y);

				/* si le bonus actuel es plus proche, on le choisi */
				if ( min > score ) {
					min = score;
					best = actuel;
				}
			}
		} else {
			this.mode = MODE_CHERCHER;
		}

		return best;
	}

	/**
	 * Cherche les bonus de toute la carte et met leur coordonnes dans this.bonusPoints
	 */
	private void chercheBonus ( ) {
		/* vide le bonusPoints */
		this.bonusPoints = new Vector();

		for ( int i=0; i < NBLIGNES; i++ ) {
			for ( int j=0; j < NBCOLONNES; j++ ) {
				/* pour chaque block de la carte,
				 * s'il est un bonus, on ajoute sa position  bonusPoints.
				 * mais on collecte que les bonus qui fonctionnent (type 4,5 et 9) */
				if ( this.niveau.carte[i][j].type == 4 || this.niveau.carte[i][j].type == 5 ||  this.niveau.carte[i][j].type == 9 ) {
					this.bonusPoints.addElement(this.niveau.carte[i][j].getPosition());
				}
			}
		}
	}

	/**
	 * Cherche la position hors de la porte des bombes, qui est accessible par le chemin le plus court
	 */
	private Point chercheSecurite ( Point p ) {

		/* on commence avec un chemin de longeur 0 et on va
		 * jusqu' la longeur ncessaire pour traverser toute la carte */
		for ( int i=0; i < NBLIGNES+NBCOLONNES; i++ ) {
			/* ce chemin peut tre utilise en direction horizontal ou en direction vertical
			 * j = part du chemin en direction x, i = part du chemin en direction y
			 * on essai toutes les combinaisons possibles */
			for ( int j=0; j <= i; j++ ) {
				if ( p.x + j < NBLIGNES && p.y + i - j < NBCOLONNES ) {
					if ( this.checkPoint(new Point(p.x + j, p.y + i - j)) ) {
						return new Point(p.x + j, p.y + i - j);
					}
				}

				if ( p.x - j > -1 && p.y - i + j > -1 ) {
					if ( this.checkPoint(new Point(p.x - j, p.y - i + j)) ) {
						return new Point(p.x - j, p.y - i + j);
					}
				}

				if ( p.x + j < NBLIGNES && p.y - i + j > -1 ) {
					if ( this.checkPoint(new Point(p.x + j, p.y - i + j)) ) {
						return new Point(p.x + j, p.y - i + j);
					}
				}

				if ( p.x - j > -1 && p.y + i - j < NBCOLONNES ) {
					if ( this.checkPoint(new Point(p.x - j, p.y + i - j)) ) {
						return new Point(p.x - j, p.y + i - j);
					}
				}
			}
		}

		/* aucune position hors de la porte des bombes trouve, alors on reste o on est */
		return p;
	}

	/**
	 * Contrle si un point est hors de la porte des bombes ou pas
	 *
	 * @param p Point  contrler
	 * @return Scurit ou pas
	 */
	private boolean checkPoint ( Point p ) {
		int bx;
		int by;
		int puissance;

		/* si on ne peut pas aller sur p, p n'est pas en scurit */
		if ( !this.gentil.testCoord(p.x, p.y) ) {
			return false;
		}

		for ( int i=0; i < this.plateau.perso.bombePosees.size(); i++ ) {
			/* on contrle pous tous les bombes : */
			bx = ((Bombe)this.plateau.perso.bombePosees.elementAt(i)).i;
			by = ((Bombe)this.plateau.perso.bombePosees.elementAt(i)).j;
			puissance = ((Bombe)this.plateau.perso.bombePosees.elementAt(i)).puissance;

			/* - est-ce que la bombe est pose  p */
			if ( by == p.y && bx == p.x ) {
				return false;
			}

			/* - est-ce que se trouve p dans la porte en direction y de la bombe */
			//if ( bx == p.x && ( (by > p.y && by - puissance >= p.y) || (by < p.y && by + puissance <= p.y)) ) {
			if ( bx == p.x && ( (by > p.y && by - puissance <= p.y) || (by < p.y && by + puissance >= p.y)) ) {
				return false;
			}

			/* - est-ce que se trouve p dans la porte en direction x de la bombe */
			//if ( by == p.y && ( (bx > p.x && bx - puissance >= p.x) || (bx < p.x && bx + puissance <= p.x)) ) {
			if ( by == p.y && ( (bx > p.x && bx - puissance <= p.x) || (bx < p.x && bx + puissance >= p.x)) ) {
				return false;
			}
		}

		/* sinon p est hors de la porte des bombes */
		return true;
	}

	/**
	 * Cherche un nouveau chemin
	 */
	private boolean chercheNouveauChemin() {

		double startZeit;
		boolean cheminTrouve = false;

		if ( DEBUG ) {
			System.out.println( this.gentil.coord.x + ", " + this.gentil.coord.y + " - " + this.target.x + ", " + this.target.y );
			startZeit = System.currentTimeMillis();
		}

		/* on enleve le vieux chemin */
		this.chemin = new Vector();

		if ( this.etoile.calculeChemin( this.gentil.coord, this.target ) ) {
			/* on y met un nouveau chemin */
			this.chemin = this.etoile.getChemin();
			this.cheminPos = 0;

			/* on charge le premier point du chemin */
			this.setProchainPoint();

			cheminTrouve = true;
		} else {
			cheminTrouve = false;
		}

		if ( DEBUG ) {
			System.out.println( System.currentTimeMillis() - startZeit );
		}

		return cheminTrouve;
	}


	/**
	 * Deplace le personnage gr
	 */
	private void deplaceAnext() {
		/* diffrence entre la position actuelle et le prochain point du chemin */
		int diffX = this.prochainPoint.x - this.gentil.coord.x;
		int diffY = this.prochainPoint.y - this.gentil.coord.y;

		/* si on ne peut pas aller sur le prochain point, on indique  chercher un nouveau chemin */
		if ( !this.gentil.testCoord( this.prochainPoint.x, this.prochainPoint.y ) ) {
			if ( DEBUG ) {
				System.out.println( "Je ne peux pas passer ? (" + this.prochainPoint.x + ", " + this.prochainPoint.y + ")" );
			}

			if ( this.mode == MODE_CACHER ) {
				this.target = this.chercheSecurite(this.gentil.coord);
			}

			this.chercheChemin = true;
		} else {

			/* si on n'est pas sur le prochain point, on s'y deplace */
			if ( diffX < 0 ) {
				this.intelligence.deplacement( "haut" );
			} else if ( diffX > 0 ) {
				this.intelligence.deplacement( "bas" );
			} else if ( diffY < 0 ) {
				this.intelligence.deplacement( "gauche" );
			} else if ( diffY > 0 ) {
				this.intelligence.deplacement( "droite" );
			} else {
				/* on est sur le prochain point */

				/* on a fait tout le chemin, on est arrive au target */
				if ( this.cheminPos >= (int) this.chemin.size() ) {
					this.intelligence.stop();

					/* si on est en mode chercher bonus
					 * (c..d. on a trouve maintenant le bonus qu'on cherche),
					 * on contrle s'il y a encore des bonus, sinon on change
					 * en mode chercher l'ennemi */
					if ( this.mode == MODE_BONUS ) {
						this.chercheBonus();

						if ( this.bonusPoints.size() == 0 ) {
							this.mode = MODE_CHERCHER;
							this.target = new Point(this.ennemi.coord.x, this.ennemi.coord.y);
						}
						
						this.chercheChemin = true;
					}

					/* si on n'est pas en train de se cacher
					 * (c..d on est maintenant en scurit),
					 * on cherche un nouveau chemin */
					if ( this.mode != MODE_CACHER ) {
						this.chercheChemin = true;
					}
				} else {
					/* on n'a pas encore fait tout le chemin, alors on charge le prochain point */
					this.intelligence.stop();
					this.setProchainPoint();

					if ( DEBUG ) {
						System.out.println( "Setting next point" );
					}
				}
			}
		}
	}


	/**
	 * Charge le prochain point du chemin
	 */
	private void setProchainPoint() {
		Block prochainBlock = (Block) ( this.chemin.elementAt( (int) ( this.chemin.size() ) - this.cheminPos - 1 ) );

		this.cheminPos++;

		this.prochainPoint = prochainBlock.getPosition();
	}

}
