package bombmapper;

import javax.swing.undo.AbstractUndoableEdit;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;

/** Change un bloc de Terrain dans un type spcifi.
 * @author Sebastian Gfeller, Andreas Jaggi
 * @created Jun 27, 2004
 * @version ChangeBlocTerrainEdit.java,v 1.2 2004/06/27 12:38:50 sgfeller Exp
 * @copyright Sebastian Gfeller, Andreas Jaggi 2004
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 */
public class ChangeBlocTerrainEdit extends AbstractUndoableEdit {
	/** Reference sur la carte */
	private Block[][] blocs;
	/** Les types originaux */
	private int[][] typesTerrainOriginal;
	/** Le type du terrain dont on change le bloc */
	private int typeTerrain;
	
	/** Coordonn x ou on commence a changer les blocs */
	private int xstart;
	/** Coordonn y ou on commence a changer les blocs */
	private int ystart;
	/** Coordonn x ou on finit a changer les blocs */
	private int xstop;
	/** Coordonn y ou on finit a changer les blocs */
	private int ystop;
	
	/** Largeur de la Carte qu'on edite */
	private int largeurCarte;
	/** Hauteur de la carte qu'on edite */
	private int hauteurCarte;
	
	/** On store une reference sur toutes les blocs. et les coordonnes
	 * du changement, le type du Terrain a changer. On store les dernieres
	 * informations sur les types des blocs dans un table et change les
	 * blocs dans le type de terrain.
	 * @param blocs reference sur la carte
	 * @param typeTerrain le type de terrain a changer le terrain
	 * @param xstart coord. x ou on commence
	 * @param ystart coord. y ou on commence
	 * @param xstop coord. x ou on finit
	 * @param ystop coord. y ou on finit
	 */
	public ChangeBlocTerrainEdit(Block[][] blocs, int typeTerrain, int xstart, int ystart, int xstop, int ystop) {
		super();
		sorterXY(xstart,ystart,xstop,ystop); // remplit les coordonns de haut  bas, gauche  droite
		this.blocs = blocs;
		this.typeTerrain = typeTerrain;
		
		// largeur et hauteur de la carte
		hauteurCarte = blocs.length;
		largeurCarte = blocs[0].length;
		
		verifierBlocs();
		changerCreerBackup();
	}
	
	/** Change la carte et fait simultanment un backup des blocs existants
	 */
	private void changerCreerBackup() {
		// calcul de grandeur de table backup
		typesTerrainOriginal = new int[ystop-ystart+1][xstop-xstart+1]; // fence post problem
		
		// on performe le backup de la partie importante: On copie une partie de la table
		for(int y=0;y<typesTerrainOriginal.length;y++) {
			for(int x=0;x<typesTerrainOriginal[y].length;x++) {
				// backup
				typesTerrainOriginal[y][x] = blocs[y+ystart][x+xstart].getType();
				// on change le type de Terrain
				blocs[y+ystart][x+xstart].setType(typeTerrain);
			}
		}
	}

	/** Verifie qu'on remplit une partie positive de la carte. Si on a dessin de droite  gauche, de haut
	 * en bas, on change les positions.
	 */
	private void sorterXY(int xstart2, int ystart2, int xstop2, int ystop2) {
			if(ystop2 >= ystart2) { // haut  bas
				ystart = ystart2;
				ystop  = ystop2;
			} else {
				ystart = ystop2;
				ystop  = ystart2;
			}
			// droite  gauche?
			if(xstop2 >= xstart2) {
				xstart = xstart2;
				xstop  = xstop2;
			} else {
				xstart = xstop2;
				xstop  = xstart2;
			}
	}

	/** Verifie si on n'est pas sur le bord de la carte -> on ne veut pas changer quelque chose l.
	 * si on est, l'action va seulement jusqu'au bord mais pas sur le bord
	 */
	private void verifierBlocs() {
		if(xstop <= 0) {
			xstop = 1;
		} else if(xstop >= (largeurCarte - 1)) {
			xstop = largeurCarte - 2;
		}
		if(ystop <= 0) {
			ystop = 1;
		} else if(ystop >= (hauteurCarte - 1)) {
			ystop = hauteurCarte - 2;
		}
		
		if (xstart <= 0) {
			xstart = 1;
		} else if (xstart >= (largeurCarte - 1)) {
			xstart = largeurCarte - 2;
		}
		if (ystart <= 0) {
			ystart = 1;
		} else if (ystart >= (hauteurCarte -1)) {
			ystart = hauteurCarte - 2;
		}
	}

	/** L'action peut-on l'annuler? */
	public boolean canUndo() { return true; }
	/** L'action peut-on la refaire? */
	public boolean canRedo() { return true; }
	/** Nom de l'action */
	public String getPresentationName() { return "Change bloc de terrain"; }
	
	/** Annule le changement en remplacant les types des blocs avec les types originaux.
	 */
	public void undo() throws CannotUndoException {
		for(int y=ystart;y<=ystop;y++) {
			for(int x=xstart;x<=xstop;x++) {
				blocs[y][x].setType(typesTerrainOriginal[y-ystart][x-xstart]);
			}
		}
	}
	
	/** Refait le changement en remplacant tout bloc dans blocs des coordonns start  stop
	 * avec le typeTerrain
	 */
	public void redo() throws CannotRedoException {
		for(int y=ystart;y<=ystop;y++) {
			for(int x=xstart;x<=xstop;x++) {
				blocs[y][x].setType(typeTerrain);
			}
		}
	}
}
