package bombmapper;

import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.undo.UndoableEdit;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;

/** L'action fait par changement des Images de terrain.
 * @author Sebastian Gfeller, Andreas Jaggi
 * @created Jun 17, 2004
 * @version ChangerImagesTerrainEdit.java,v 1.9 2004/06/27 09:55:25 sgfeller Exp
 * @copyright Sebastian Gfeller, Andreas Jaggi 2004
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 */
public class ChangerImagesTerrainEdit implements UndoableEdit,ActionListener {
	private String[] nomsFichiersImages;
	private String[] nomsFichiersImagesOriginal;
	private String[] nomsFichiersImagesChanges;
	private Image[] imagesTerrain;
	private int[] imagesChoisis;
	private int[] imagesChoisisChange;
	private int[] imagesChoisisOriginal;
	
	private JarImageManager managerImagesTerrain;
	private JRadioButton[][] selectionImage = new JRadioButton[4][];
	
	private JButton bAnnuler;
	private JButton bChanger;
	
	private LevelView fenetreCible;
	
	// Le Layout
	JFrame fenetreChoix;

	/** Ouvre un JFrame avec les blocs  choisir (liste des
	 * Images possibles gner automatiquement
	 */
	public ChangerImagesTerrainEdit(String[] nomsFichiersImages, Image[] imagesTerrain, LevelView fenetreCible) throws IOException {
		super();

		this.nomsFichiersImages = nomsFichiersImages;
		this.imagesTerrain = imagesTerrain;
		this.fenetreCible = fenetreCible;
		// Pour pouvoir annuler l'action plus tard
		// on ne change pas les images bonus, alors moins a stocker
		// dans la table
		nomsFichiersImagesOriginal = new String[4];
		nomsFichiersImagesChanges = new String[4];
		for(int i=0;i<4;i++) {
			nomsFichiersImagesOriginal[i] = this.nomsFichiersImages[i];
		}
		imagesChoisis = new int[4];
		
		/* appelle un manager qui trouve les fichiers images
		 * dans ce jar.
		 */
		managerImagesTerrain = new JarImageManager(nomsFichiersImagesOriginal);

		// Partie GUI
		fenetreChoix = new JFrame("Choix des Images:");
		Container cont = fenetreChoix.getContentPane();
		cont.setLayout(new GridLayout(10,1));
		
		// Alors, on remplit la fentre avec differents JPanels pour les differents Terrains
		JPanel pTitre = new JPanel();
		JLabel lTitre = new JLabel("Choisir un seul type d'\u00E9l\u00E9ments dans chaque liste:");
		pTitre.add(lTitre);
		cont.add(pTitre);
		
		JPanel pTTypesSols = new JPanel();
		JLabel lTypesSols = new JLabel("types des sols");
		pTTypesSols.add(lTypesSols);
		cont.add(pTTypesSols);
		
		JPanel pTypesSols = new JPanel();
		populerChoixTerrain(pTypesSols,"fond/sol",0);
		cont.add(pTypesSols);
		
		JPanel pTTypesPierresNulles = new JPanel();
		JLabel lTypesPierresNulles = new JLabel("Types des pierres nulles");
		pTTypesPierresNulles.add(lTypesPierresNulles);
		cont.add(pTTypesPierresNulles);
		
		JPanel pTypesPierresNulles = new JPanel();
		populerChoixTerrain(pTypesPierresNulles,"fond/mou",1);
		cont.add(pTypesPierresNulles);
		
		JPanel pTTypesPierresDures = new JPanel();
		JLabel lTypePierresDures = new JLabel("Types de pierres dures");
		pTTypesPierresDures.add(lTypePierresDures);
		cont.add(pTTypesPierresDures);
		
		JPanel pTypesPierresDures = new JPanel();
		populerChoixTerrain(pTypesPierresDures,"fond/mur",2);
		cont.add(pTypesPierresDures);
		
		JPanel pTTypesElementsIndestructibles = new JPanel();
		JLabel lTypesElementsIndestructibles = new JLabel("Types d'\u00E9l\u00E9ments indestructibles");
		pTTypesElementsIndestructibles.add(lTypesElementsIndestructibles);
		cont.add(pTTypesElementsIndestructibles);
		
		JPanel pTypesElementsIndestructibles = new JPanel();
		populerChoixTerrain(pTypesElementsIndestructibles,"fond/indestructible",3);
		cont.add(pTypesElementsIndestructibles);
		
		JPanel pChanger = new JPanel();
		bAnnuler = new JButton("Annuler");
		bChanger = new JButton("Changer images");
		bAnnuler.addActionListener(this);
		bChanger.addActionListener(this);
		pChanger.add(bAnnuler);
		pChanger.add(bChanger);
		cont.add(pChanger);

		fenetreChoix.pack();
		fenetreChoix.setVisible(true);
	}
	
	/** Cette methode ajoute des images et des boutons radio au
	 * panel spcifi, un bouton pour tout fichier image
	 * specifi dans imagesDansLaRepertoire. C'est mal fai, mais
	 * d'une autre perspective, les fichiers sont stors
	 * dans le jar, alors a ne change pas?
	 * 
	 * @param pChoix Le panel ou on ajoute la choix
	 * @param chemin Le chemin sur des images
	 * @throws IOException
	 */
	private void populerChoixTerrain(JPanel pChoix,String chemin, int pos) {
		// avec un FlowLayout, ajouter les images avec des boutons au JPanel
		ImageIcon[] imagesDansLaRepertoire = managerImagesTerrain.getImagesDuType(pos);
		pChoix.setLayout(new FlowLayout());
		JLabel[] terrainAssocie = new JLabel[imagesDansLaRepertoire.length];
		selectionImage[pos] = new JRadioButton[imagesDansLaRepertoire.length];
		ButtonGroup groupeBoutons = new ButtonGroup();
		
		for(int i=0;i<imagesDansLaRepertoire.length;i++) {
			/* On cre un ImageIcon avec l'image du Terrain.
			 * Ceci est le contenu du JLabel qui est associ
			 * au bouton cre.
			 */
			terrainAssocie[i] = new JLabel(imagesDansLaRepertoire[i]);
			selectionImage[pos][i] = new JRadioButton();
			selectionImage[pos][i].addActionListener(this);
			terrainAssocie[i].setLabelFor(selectionImage[pos][i]);
			pChoix.add(terrainAssocie[i]);
			pChoix.add(selectionImage[pos][i]);
			groupeBoutons.add(selectionImage[pos][i]);
			
			// voir si c'est l'image ancien
			if(managerImagesTerrain.getNomsImagesDuType(pos)[i].endsWith(nomsFichiersImages[pos])) {
				selectionImage[pos][i].setSelected(true);
				imagesChoisis[pos] = i;
			}
		}
		
		// Si on ne change rien aprs, le terrain reste le mme.
		imagesChoisisOriginal = imagesChoisis;
		imagesChoisisChange   = imagesChoisis;
	}
	
	// Proprits de cette action
	public boolean canRedo() { return true; }
	public boolean canUndo() { return true; }
	public String getPresentationName() { return "Changer Images de Terrain"; }
	
	/** Transferre les images  levelView. */
	private void mettreImagesAJour() {
		for(int i=0;i<4;i++) {
			imagesTerrain[i] = managerImagesTerrain.getImage(i).getMonImage().getImage();
		}
	}
	
	/** Annule l'action de changement des images	 */
	public void undo() throws CannotUndoException {
		imagesChoisisChange = imagesChoisis;
		for(int i=0;i<4;i++) {
			nomsFichiersImages[i] = nomsFichiersImagesOriginal[i];
			imagesChoisis = imagesChoisisOriginal;
		}
		mettreImagesAJour();
	}

	/** Refait le changement des images */
	public void redo() throws CannotRedoException {
		for(int i=0;i<4;i++) {
			nomsFichiersImages[i] = nomsFichiersImagesChanges[i];
			imagesChoisis = imagesChoisisChange;
		}
		mettreImagesAJour();
	}

	public void die() { }
	public boolean addEdit(UndoableEdit arg0) { return false; }
	public boolean replaceEdit(UndoableEdit arg0) { return false; }
	public boolean isSignificant() { 	return false; }
	public String getUndoPresentationName() { return null; }
	public String getRedoPresentationName() { return null; }

	/** L'action perform: Gre les actions de changement dans les boutons radio et dans
	 * les boutons Annuler, Changer
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		for(int y=0;y<selectionImage.length;y++) {
			for(int x=0;x<selectionImage[y].length;x++) {
				if(e.getSource() == selectionImage[y][x]) {
					nomsFichiersImagesChanges[y] = managerImagesTerrain.getNomsImagesDuType(y)[x];
					imagesChoisis[y] = x;
					// managerImagesTerrain.setImageChoisi(y,x);
				}
			}
		}
		
		if(e.getSource() == bAnnuler) { // on annule le changement
			fenetreChoix.setVisible(false);
			fenetreChoix.dispose();
			fenetreChoix = null;
		} else if (e.getSource() == bChanger) { // On veut changer les images
			// La, c'est l'action
			for(int i=0;i<4;i++) {
				nomsFichiersImages[i] = nomsFichiersImagesChanges[i];
				managerImagesTerrain.setImageChoisi(i,imagesChoisis[i]);
			}
			mettreImagesAJour();
			fenetreChoix.setVisible(false);
			fenetreChoix.dispose();
			fenetreChoix = null;
			
			fenetreCible.repaint();
		}
	}
}
