package bombmapper;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import javax.swing.ImageIcon;

/** Manager pour les fichiers images dans le JAR.
 * @author Sebastian Gfeller, Andreas Jaggi
 * @created Jun 22, 2004
 * @version JarImageManager.java,v 1.5 2004/06/23 13:07:42 sgfeller Exp
 * @copyright Sebastian Gfeller, Andreas Jaggi 2004
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 */
public class JarImageManager {
	/** Le fichier Jar dont cette classe est dedans. */
	private JarFile jf;
	private ArrayList jarEntries;
	/** C'est toujours "/", parce que les fichiers level sont fait dans ce format */
	private String delimiteurChemins = "/";
	/** Alors, la methode n'est plus indpendant du nom du fichier JAR. Mais c'est *beaucoup*
	 * plus simple!
	 */
	private String cheminDuJar = "bomberman.jar";
	/** Les images choisis */
	private int[] imagesChoisis;
	private ImageResource[][] imagesDansLesRepertoires;
	
	
	/** Appel avec les noms des fichiers utiliss a ce moment,
	 * le constructeur construit une liste avec tous les images
	 * trouvs dans les repertoires des images choisis.
	 * Il store les images alors dans un tableau Image[] qui peut tre
	 * retourn.
	 * @throws IOException
	 */
	public JarImageManager(String[] fichiersImagesChoisis) throws IOException {
		super();
		jf = new JarFile(cheminDuJar);
		
		imagesDansLesRepertoires = new ImageResource[fichiersImagesChoisis.length][];
		for(int i=0;i<imagesDansLesRepertoires.length;i++) {
			imagesDansLesRepertoires[i] = trouverToutFichierImage(fichiersImagesChoisis[i]);
		}
		
		imagesChoisis = new int[fichiersImagesChoisis.length];
	}

	/** Trouve tout fichier Image dans le chemin du fichier
	 * qui est donn par parametre. On cherche dans le JAR et
	 * utilise le FiltreFichiersImagesBlocs pour seulement retourner
	 * des blocs desirs
	 * 
	 * @param cheminFichiers chemin d'un fichier dans la repertoire qui doit tre cherche
	 * @return table avec tous les images trouvs dans la repertoire
	 */
	private ImageResource[] trouverToutFichierImage(String cheminFichiers) throws IOException {
		FiltreFichiersImagesBlocs filtre = new FiltreFichiersImagesBlocs();
		/* On extrait le repertoire */
		String nomRepertoireImages = cheminFichiers.substring(0,cheminFichiers.lastIndexOf(delimiteurChemins));
		
		jarEntries = new ArrayList();
		
		Enumeration e = jf.entries();
		
		while(e.hasMoreElements()) {
			JarEntry je = (JarEntry)e.nextElement();
			if(je.toString().startsWith(nomRepertoireImages) && filtre.accept(je.toString())) {
				jarEntries.add(je);
			}
		}

		// toutes les elements commencant avec cheminFichiers, dans format ImageIcon
		ImageResource[] imagesTrouves = new ImageResource[jarEntries.size()];
		for(int i=0;i<imagesTrouves.length;i++) {
			imagesTrouves[i] = new ImageResource(jarEntries.get(i).toString());
		}
		return (imagesTrouves);
	}
	
	/** Retourne un table avec les images d'un type. */
	public ImageIcon[] getImagesDuType(int type) {
		ImageIcon[] imagesRetournes = new ImageIcon[imagesDansLesRepertoires[type].length];
		for(int i=0;i<imagesRetournes.length;i++) {
			imagesRetournes[i] = imagesDansLesRepertoires[type][i].getMonImage();
		}
		return imagesRetournes;
	}
	

	public String[] getNomsImagesDuType(int type) {
		String[] cheminsRetournes = new String[imagesDansLesRepertoires[type].length];
		for(int i=0;i<cheminsRetournes.length;i++) {
			cheminsRetournes[i] = imagesDansLesRepertoires[type][i].getMonChemin();
		}
		return cheminsRetournes;
	}
	
	/** Pour une ligne, definie l'image choisi.
	 * @param ligne la ligne (type du terrain)
	 * @param index position de l'image dans l'array getImagesDuType()
	 */
	public void setImageChoisi(int ligne, int index) {
		imagesChoisis[ligne] = index;
	}
	
	public ImageResource getImage(int ligne) {
		return imagesDansLesRepertoires[ligne][imagesChoisis[ligne]];
	}
}
